/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package de.dlr.ts.utils.xmladmin2;

/**
 *
 * @author <a href="mailto:maximiliano.bottazzi@dlr.de">Maximiliano Bottazzi</a>
 */
public class MalformedKeyOrNameException extends Exception
{

    public MalformedKeyOrNameException()
    {
    }

    public MalformedKeyOrNameException(String message)
    {
        super(message);
    }
    
    
}
