const std::string opendriveTypemap =
"<types xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/types_file.xsd\">\n"
"    <type id=\"driving\"       priority=\"1\"  speed=\"13.89\" width=\"3.65\" disallow=\"pedestrian rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"stop\"          priority=\"1\"  speed=\"13.89\" width=\"3.65\" disallow=\"pedestrian rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"restricted\"    priority=\"0\"  speed=\"13.89\" width=\"3.65\" disallow=\"all\"/> <!-- ambiguous semantics. Could mean bus-lane as well as fully-area restricted (but surfaced) lane area (examples from http://www.opendrive.org -->\n"
"    <type id=\"parking\"       priority=\"1\"  speed=\"1.39\"  width=\"2.50\" disallow=\"pedestrian rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"entry\"         priority=\"1\"  speed=\"22.22\" width=\"3.65\" disallow=\"pedestrian rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"exit\"          priority=\"1\"  speed=\"22.22\" width=\"3.65\" disallow=\"pedestrian rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"offRamp\"       priority=\"1\"  speed=\"22.22\" width=\"3.65\" disallow=\"pedestrian bicycle moped rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"onRamp\"        priority=\"1\"  speed=\"22.22\" width=\"3.65\" disallow=\"pedestrian bicycle moped rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"roadWorks\"     priority=\"0\"  speed=\"1.39\"  width=\"3.65\" allow=\"authority\"/>\n"
"    <type id=\"tram\"          priority=\"2\"  speed=\"13.89\" width=\"3.65\" allow=\"tram\"/>\n"
"    <type id=\"rail\"          priority=\"3\"  speed=\"33.33\" width=\"3.65\" allow=\"rail rail_urban rail_electric\"/>\n"
"\n"
"    <!-- should be enabled via additional typemaps -->\n"
"    <!--\n"
"    <type id=\"biking\"        priority=\"-1\" speed=\"5.56\"  width=\"1.50\" allow=\"bicycle\"      />\n"
"    <type id=\"sidewalk\"      priority=\"-2\" speed=\"2.78\"  width=\"2.50\" allow=\"pedestrian\"   />\n"
"    -->\n"
"\n"
"    <!-- discarded -->\n"
"    <type id=\"none\"          priority=\"0\"  speed=\"1.39\" width=\"1.0\"   disallow=\"all\"          discard=\"true\"/> <!-- non-lane spacing. gives ugly junction shapes when present on the outside of edges (as seen in official examples) -->\n"
"    <type id=\"shoulder\"      priority=\"0\"  speed=\"1.39\" width=\"1.0\"   disallow=\"all\"          discard=\"true\"/>\n"
"    <type id=\"border\"        priority=\"0\"  speed=\"1.39\" width=\"0.1\"   disallow=\"all\"          discard=\"true\"/>\n"
"    <type id=\"median\"        priority=\"0\"  speed=\"1.39\" width=\"0.1\"   disallow=\"all\"          discard=\"true\"/>\n"
"    <type id=\"special1\"      priority=\"1\"  speed=\"22.22\" width=\"3.65\" allow=\"custom1\"         discard=\"true\"/>\n"
"    <type id=\"special2\"      priority=\"1\"  speed=\"22.22\" width=\"3.65\" allow=\"custom2\"         discard=\"true\"/>\n"
"    <type id=\"special3\"      priority=\"1\"  speed=\"22.22\" width=\"3.65\" allow=\"custom1 custom2\" discard=\"true\"/>\n"
"\n"
"    <!-- deprecated -->\n"
"    <type id=\"mwyEntry\"      priority=\"1\"  speed=\"22.22\" width=\"3.65\" disallow=\"pedestrian bicycle moped rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"mwyExit\"       priority=\"1\"  speed=\"22.22\" width=\"3.65\" disallow=\"pedestrian bicycle moped rail rail_urban rail_electric tram ship\"/>\n"
"\n"
"    <!-- not fully implemented -->\n"
"    <type id=\"bidirectional\" priority=\"1\"  speed=\"1.39\"  width=\"3.65\" disallow=\"pedestrian rail rail_urban rail_electric tram ship\"/> \n"
"</types>\n"
;
const std::string osmTypemap =
"<types xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/types_file.xsd\">\n"
"    <type id=\"highway.motorway\"       numLanes=\"2\" speed=\"44.44\" priority=\"13\" oneway=\"true\"  disallow=\"pedestrian bicycle moped rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.motorway_link\"  numLanes=\"1\" speed=\"22.22\" priority=\"12\" oneway=\"true\"  disallow=\"pedestrian bicycle moped rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.trunk\"          numLanes=\"2\" speed=\"27.78\" priority=\"11\" oneway=\"false\" disallow=\"pedestrian bicycle rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.trunk_link\"     numLanes=\"1\" speed=\"22.22\" priority=\"10\" oneway=\"false\" disallow=\"pedestrian bicycle rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.primary\"        numLanes=\"2\" speed=\"27.78\" priority=\"9\"  oneway=\"false\" disallow=\"rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.primary_link\"   numLanes=\"1\" speed=\"22.22\" priority=\"8\"  oneway=\"false\" disallow=\"rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.secondary\"      numLanes=\"2\" speed=\"27.78\" priority=\"7\"  oneway=\"false\" disallow=\"rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.secondary_link\" numLanes=\"1\" speed=\"22.22\" priority=\"6\"  oneway=\"false\" disallow=\"rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.tertiary\"       numLanes=\"1\" speed=\"22.22\" priority=\"6\"  oneway=\"false\" disallow=\"rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.tertiary_link\"  numLanes=\"1\" speed=\"22.22\" priority=\"5\"  oneway=\"false\" disallow=\"rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.unclassified\"   numLanes=\"1\" speed=\"13.89\" priority=\"5\"  oneway=\"false\" disallow=\"rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.residential\"    numLanes=\"1\" speed=\"13.89\" priority=\"4\"  oneway=\"false\" disallow=\"rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.living_street\"  numLanes=\"1\" speed=\"2.78\"  priority=\"3\"  oneway=\"false\" disallow=\"rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.service\"        numLanes=\"1\" speed=\"5.56\"  priority=\"2\"  oneway=\"false\" allow=\"delivery pedestrian bicycle\"/>\n"
"    <type id=\"highway.track\"          numLanes=\"1\" speed=\"5.56\"  priority=\"1\"  oneway=\"false\" disallow=\"rail rail_urban rail_electric tram ship\"/>\n"
"    <type id=\"highway.unsurfaced\"     numLanes=\"1\" speed=\"8.33\"  priority=\"1\"  oneway=\"false\" disallow=\"rail rail_urban rail_electric tram ship\"/>\n"
"\n"
"    <!-- everything which serves mainly pedestrians is oneway because all current pedestrian models do not care about direction -->\n"
"    <type id=\"highway.footway\"        numLanes=\"1\" speed=\"2.78\"  priority=\"1\"  oneway=\"true\"  width=\"2\" allow=\"pedestrian\"/>\n"
"    <type id=\"highway.pedestrian\"     numLanes=\"1\" speed=\"2.78\"  priority=\"1\"  oneway=\"true\"  width=\"2\" allow=\"pedestrian\"/>\n"
"    <type id=\"highway.path\"           numLanes=\"1\" speed=\"2.78\"  priority=\"1\"  oneway=\"true\"  width=\"2\" allow=\"pedestrian bicycle\"/>\n"
"    <type id=\"highway.bridleway\"      numLanes=\"1\" speed=\"2.78\"  priority=\"1\"  oneway=\"true\"  width=\"2\" allow=\"pedestrian\"/>\n"
"    <type id=\"highway.cycleway\"       numLanes=\"1\" speed=\"8.33\"  priority=\"1\"  oneway=\"false\" width=\"1\" allow=\"bicycle\"/>\n"
"    <type id=\"highway.step\"           numLanes=\"1\" speed=\"1.39\"  priority=\"1\"  oneway=\"true\"  width=\"2\" allow=\"pedestrian\"/>\n"
"    <type id=\"highway.steps\"          numLanes=\"1\" speed=\"1.39\"  priority=\"1\"  oneway=\"true\"  width=\"2\" allow=\"pedestrian\"/>\n"
"    <type id=\"highway.stairs\"         numLanes=\"1\" speed=\"1.39\"  priority=\"1\"  oneway=\"true\"  width=\"2\" allow=\"pedestrian\"/>\n"
"\n"
"    <type id=\"highway.bus_guideway\"   numLanes=\"1\" speed=\"27.78\" priority=\"1\"  oneway=\"true\"  allow=\"bus\"/>\n"
"    <type id=\"highway.raceway\"        numLanes=\"2\" speed=\"83.33\" priority=\"14\" oneway=\"false\" allow=\"vip\"/>\n"
"    <type id=\"highway.ford\"           numLanes=\"1\" speed=\"2.78\"  priority=\"1\"  oneway=\"false\" allow=\"army\"/>\n"
"\n"
"    <type id=\"railway.rail\"           numLanes=\"1\" speed=\"83.33\" priority=\"15\" oneway=\"true\"  allow=\"rail rail_electric\"/>\n"
"    <type id=\"railway.tram\"           numLanes=\"1\" speed=\"13.89\" priority=\"15\" oneway=\"true\"  allow=\"tram\"/>\n"
"    <type id=\"railway.light_rail\"     numLanes=\"1\" speed=\"27.78\" priority=\"15\" oneway=\"true\"  allow=\"rail_urban\"/>\n"
"    <type id=\"railway.subway\"         numLanes=\"1\" speed=\"27.78\" priority=\"15\" oneway=\"true\"  allow=\"rail_urban\"/>\n"
"    <type id=\"railway.preserved\"      numLanes=\"1\" speed=\"27.78\" priority=\"15\" oneway=\"true\"  allow=\"rail\"/>\n"
"\n"
"</types>\n"
;
